clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');

%% Description of the example

% This example shows the effects of time warping on style of walking
% motions when two motions are matched. First, the motions are show with
% original timings and then the same motions warped to matched timings.
% Visually, time warping can be seen to create slowed down and fast
% forwarded appearance. Also, acceleration artifacts can appear if the
% steps of the motions vary in duration.
%
% To see the code that finds matching steps between walking motions, see
% file script_1_load_and_warp_walks.m.

%% Warping motions to same phase

load('matlab_data/walks_a_warped.mat'); % Run script_1 to create this file

index_a = 2;
index_b = 6;

ma = walk_motions_a(index_a);
mb = walk_motions_a(index_b);

% Adding motion with original timings
ratio_a = -1;
ratio_b = -1;
bvhdata_a{1} = ma.bvhdata(1:ma.time_this(end), :);
bvhdata_b{1} = mb.bvhdata(1:mb.time_this(end), :);

% Producing versions where the timings are matched by warping the motions
% to same timing that is one of the originals or a weighted mean of the
% original timings.
for ratio = [0 0.5 1]
    ratio_a(end+1) = ratio;
    ratio_b(end+1) = 1-ratio;
    desired_timing = (ma.time_this * ratio)  + (mb.time_this * (1-ratio));
    time_transfer_a = interp1(desired_timing(:), ma.time_this(:), 1:desired_timing(end), 'linear');
    time_transfer_b = interp1(desired_timing(:), mb.time_this(:), 1:desired_timing(end), 'linear');
    bvhdata_a{end+1} = f_warp_motion(time_transfer_a, ma);
    bvhdata_b{end+1} = f_warp_motion(time_transfer_b, mb);
end

%% Visualization

% Width of the motion trajectories shows velocities

for i1 = 1 : length(bvhdata_a)
    coordinates_a = f_fast_bvh_to_coordinates(bvhdata_a{i1}, ma.skel);
    coordinates_b = f_fast_bvh_to_coordinates(bvhdata_b{i1}, mb.skel);
    
    % Setting the characters to walk side by side
    offset = repmat([150 0 0], size(coordinates_a, 1), 24);
    coordinates_a = coordinates_a + offset;
    bvh_a = bvhdata_a{i1};
    bvh_a(:, 1:3) = bvh_a(:, 1:3) + offset(:, 1:3);
    bvh_b = bvhdata_b{i1};
    
    f_animate_pair(bvh_b, bvh_a, mb.skel, ma.skel, coordinates_b, coordinates_a, 'figNum', 100 + i1);
    if (ratio_b(i1) == -1)
        title('original timings');
    else
        title(['timing: left ' num2str(ratio_b(i1) * 100) '%, right ' num2str(ratio_a(i1)*100) '%']);
    end
    pause(0.5);
end