clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');
addpath('./external_libraries/slerp');

%% Description of the example

% This example demonstrates pairwise interpolation where the motions are
% time warped to same duration without any attempt to match their timings
% otherwise. First is shown a case of hand waving, where the interpolation
% succeeds in creating a blend of styles. A second case demonstrates an
% interpolation between two walking sequences where the simple time warping
% turns the motions to an unnatural looking dance.

%% Interpolation (good result)

load('matlab_data/waves_1.mat'); % Run script_2 to create this file

from_index = 32;
to_index = 35;

percentages = 0.0 : 0.5 : 1.0;

index = 0;
for percent = percentages
    index = index + 1;
    bvhdata{index} = f_interpolation_pairwise(wave_motions_a(from_index), wave_motions_a(to_index), percent);
    coordinates{index} = f_fast_bvh_to_coordinates(bvhdata{index}(1:end, :), wave_motions_a(1).skel);
end

% Visualization
for i1 = 1:length(coordinates)
    f_animate(bvhdata{i1}, wave_motions_a(1).skel, coordinates{i1}, 'figNum', 100+i1);
    title(['Interpolation: motion A ' num2str(percentages(i1) * 100) '%, motion B ' num2str((1-percentages(i1))*100) '%']);
    pause(1.0);
end

%% Interpolation (bad result)

load('matlab_data/walks_a_warped.mat');

from_index = 1;
to_index = 5;

percentages = 0.0 : 0.5 : 1.0;

index = 0;
for percent = percentages
    index = index + 1;
    bvhdata_2{index} = f_interpolation_pairwise(walk_motions_a(from_index), walk_motions_a(to_index), percent);
    coordinates2{index} = f_fast_bvh_to_coordinates(bvhdata_2{index}(1:end, :), walk_motions_a(1).skel);
end

% Visualization
for i1 = 1:length(coordinates2)
    f_animate(bvhdata_2{i1}, walk_motions_a(1).skel, coordinates2{i1}, 'figNum',  200+i1 );
    title(['Interpolation: motion A ' num2str(percentages(i1) * 100) '%, motion B ' num2str((1-percentages(i1))*100) '%']);
    pause(1.0);
end