clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');

%% Description of the example

% This example demonstrates interpolation between several walking motions
% that have been carefully selected and time warped to match well. First is
% shown an interpolation between the same walks that were not interpolated
% well by the pairwise implementation (ex_02_interpolation_pairwise.m).
% After that is shown an equal interpolation between all the 35 walks.

%% Interpolation between two motion

load('matlab_data/walks_a_warped.mat'); % Run script_1 to create this file

from_index = 1;
to_index = 5;
num_of_interpolations = 1; % minimum is 1

parameters = zeros(num_of_interpolations+2, 35);
parameters(:, from_index) = (1.0 : -1/(1+num_of_interpolations) : 0.0)';
parameters(:, to_index) =   (0.0 :  1/(1+num_of_interpolations) : 1.0)'; 

for i1 = 1:num_of_interpolations+2
    bvhdata{i1} = f_interpolation_nway(parameters(i1, :), walk_motions_a);
    coordinates{i1} = f_fast_bvh_to_coordinates(bvhdata{i1}(1:end, :), walk_motions_a(i1).skel);
end

% Visualization
for i1 = 1:length(coordinates)
    f_animate(bvhdata{i1}, walk_motions_a(1).skel, coordinates{i1}, 'figNum', 100+i1);
    title(['Interpolation: motion A ' num2str(parameters(i1, from_index) * 100) ...
        '%, motion B ' num2str((parameters(i1, to_index))*100) '%']);
    pause(1.0);
end

%% Average of all 35 motions

param = ones(1, 35) / 35;

bvhdata_2 = f_interpolation_nway(param, walk_motions_a);
coordinates2 = f_fast_bvh_to_coordinates(bvhdata_2(1:end, :), walk_motions_a(1).skel);

% Visualization
f_animate(bvhdata_2 , walk_motions_a(1).skel, coordinates2, 'figNum', 200);
title('Even interpolation between 35 walking motions');
