clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');

%% Description of the example

% This example demonstrates pairwise motion extrapolation. First case shows 
% a successful extrapolation between two hand waves. (0% and 100% are
% original motions and over 100% are extrapolations.) Second case shows a
% bad result from extrapolation where the characters has unnatural poses
% and body parts go through each other.

%% Extrapolation (good result)

load('matlab_data/waves_1.mat'); % Run script_2 to create this file

from_index = 10;
to_index = 1;

percentages = 0.0 : 0.5 : 2.0;

index = 0;
for percent = percentages
    index = index + 1;
    bvhdata{index} = f_extrapolation_pairwise(wave_motions_a(from_index), wave_motions_a(to_index), percent);
    coordinates{index} = f_fast_bvh_to_coordinates(bvhdata{index}(1:end, :), wave_motions_a(1).skel);
end

% Visualization
for i1 = 1:length(coordinates)
    f_animate(bvhdata{i1}, wave_motions_a(1).skel, coordinates{i1}, 'figNum', 100);
    title(['Extrapolation: ' num2str(percentages(i1) * 100) '%']);
    pause(1.0);
end

%% Extrapolation (bad result)

from_index = 32;
to_index = 35;

percentages = 0.0 : 0.5 : 2.0;

index = 0;
for percent = percentages
    index = index + 1;
    bvhdata{index} = f_extrapolation_pairwise(wave_motions_a(from_index), wave_motions_a(to_index), percent);
    coordinates{index} = f_fast_bvh_to_coordinates(bvhdata{index}(1:end, :), wave_motions_a(1).skel);
end

% Visualization
for i1 = 1:length(coordinates)
    f_animate(bvhdata{i1}, wave_motions_a(1).skel, coordinates{i1}, 'figNum', 200);
    title(['Extrapolation: ' num2str(percentages(i1) * 100) '%']);
    pause(1.0);
end