clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');

%% Description of the example

% This example demonstrates style transfer where a style is defined as a
% difference between two motions. The difference signal can then be scaled
% and transferred to a third motion. In the first case, the style
% difference is from a regular walk to a more aggressive and faster walk.
% The style is successfully transferred to a walk with a depressed posture.
% The second case takes style difference from a fast walk to a slower walk
% and transfers this to an aggressive walk. In the second case, artifacts
% such as feet slipping backwards, unnaturally slow segments and body parts
% going through each other appear.

%% 

load('matlab_data/walks_a_warped.mat'); % Run script_1 to create this file

%% Style transfer (good result)

transfer_percents = [ 0.0 0.5 1.0 1.5 ];
from_index = 3;
to_index = 5;
new_start_index = 6;

for i1 = 1 : length(transfer_percents)
    bvhdata{i1} = f_style_transfer( ...
        walk_motions_a(from_index), ...
        walk_motions_a(to_index),  ...
        walk_motions_a(new_start_index), ...
        transfer_percents(i1));
    coordinates{i1} = f_fast_bvh_to_coordinates(bvhdata{i1}(1:end, :), walk_motions_a(1).skel);
end

% Visualizing style difference in original motions
offset = repmat([-150 0 0], size( walk_motions_a(from_index).coordinates, 1), 24);
    coordinates_a = walk_motions_a(from_index).coordinates + offset;
    motion_a = walk_motions_a(from_index).bvhdata;
    motion_a(:, 1:3) = walk_motions_a(from_index).bvhdata(:, 1:3) + offset(:, 1:3);
 f_animate_pair(...
     motion_a, ...
     walk_motions_a(to_index).bvhdata, ...
     walk_motions_a(from_index).skel,...
     walk_motions_a(to_index).skel, ...
     coordinates_a, ...
     walk_motions_a(to_index).coordinates, ...
     'figNum', 100);
title('Style difference from the motion on the left to one on the right');
pause(1.0);
 
% Visualization of style transfer
for i1 = 1 : length(transfer_percents)
    f_animate(bvhdata{i1}, walk_motions_a(1).skel, coordinates{i1}, 'figNum', 101);
    title(['Style transfer amount: ' num2str(transfer_percents(i1) * 100) '%']);
    pause(1.0);
end

%% Style transfer (bad result)

transfer_percents = [ 0.0 0.5 1.0 1.5 ];
from_index = 2;
to_index = 3;
new_start_index = 5;

for i1 = 1 : length(transfer_percents)
    bvhdata{i1} = f_style_transfer( ...
        walk_motions_a(from_index), ...
        walk_motions_a(to_index),  ...
        walk_motions_a(new_start_index), ...
        transfer_percents(i1));
    coordinates{i1} = f_fast_bvh_to_coordinates(bvhdata{i1}(1:end, :), walk_motions_a(1).skel);
end

% Visualizing style difference in original motions
offset = repmat([-150 0 0], size( walk_motions_a(from_index).coordinates, 1), 24);
    coordinates_a = walk_motions_a(from_index).coordinates + offset;
    motion_a = walk_motions_a(from_index).bvhdata;
    motion_a(:, 1:3) = walk_motions_a(from_index).bvhdata(:, 1:3) + offset(:, 1:3);
 f_animate_pair(...
     motion_a, ...
     walk_motions_a(to_index).bvhdata, ...
     walk_motions_a(from_index).skel,...
     walk_motions_a(to_index).skel, ...
     coordinates_a, ...
     walk_motions_a(to_index).coordinates, ...
     'figNum', 200);
title('Style difference from the motion on the left to one on the right');
pause(1.0);

% Visualization of style transfer
for i1 = 1 : length(transfer_percents)
    f_animate(bvhdata{i1}, walk_motions_a(1).skel, coordinates{i1}, 'figNum', 201);
    title(['Style transfer amount: ' num2str(transfer_percents(i1) * 100) '%']);
    pause(1.0);
end
