clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');

%% Description of the example

% This example demonstrates the use of N-way extrapolation. First is shown
% a case where several extrapolations and style transfers are used to add
% more styles to a motion. Second case shows an extrapolation between a
% regular and an aggressive motion that produces reasonable exaggerations
% towards more aggressive style, but turns unnatural looking when the style
% is extrapolated towards more regular versions.

%% 

load('matlab_data/walks_a_warped.mat'); % Run script_1 to create this file


%% Adjusting style with combined extrapolations and style transfers

clear parameters;
  
parameters(:, 1) = [0    0 0    1   0   0   0    0 0 0 0   0 0 0 0 0 0 0 0 0   0 0 0 0    0 0 0 0 0 0 0 0 0 0 0 ];
parameters(:, 2) = [-0.7 0 0    1.7 0   0   0    0 0 0 0   0 0 0 0 0 0 0 0 0   0 0 0 0    0 0 0 0 0 0 0 0 0 0 0 ];
parameters(:, 3) = [-0.7 0 -0.4 1.7 0.4 0   0    0 0 0 0   0 0 0 0 0 0 0 0 0   0 0 0 0    0 0 0 0 0 0 0 0 0 0 0 ];
parameters(:, 4) = [-0.7 0 -0.9 1.7 0.1 0   0.55 0 0 0 0   0 0 0 0 0 0 0 0 0   0 0 0 0.25 0 0 0 0 0 0 0 0 0 0 0 ];
parameters(:, 5) = [-0.7 0 -1.0 0.8 0.1 0.2 0.55 0 0 0 0.4 0 0 0 0 0 0 0 0 0.4 0 0 0 0.25 0 0 0 0 0 0 0 0 0 0 0 ];

for i1 = 1 : size(parameters, 2);
    bvhdata{i1} = f_extrapolation_nway(parameters(:, i1), walk_motions_a);
    coordinates{i1} = f_fast_bvh_to_coordinates(bvhdata{i1}(1:end, :), walk_motions_a(1).skel);
end

story = {
    'A man has a regular day'
    'He gets a bit drunk in a bar'
    'He gets aggressive while drunk'
    'He hurts his leg while angry'
    'He gets depressed by the injured leg'
    };

% Visualization
for i1 = 1 : length(bvhdata);
    figure(100);
    clf;
    title(story{i1});
    pause(1.0);
    f_animate(bvhdata{i1}, walk_motions_a(1).skel, coordinates{i1}, 'figNum', 100, 'titleText', story{i1});
    pause(1.0);
end

%% Case where extrapolation produces unexpected results

clear parameters;
  
parameters(:, 1) = [0.5  0 0 0 0.5  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 ];
parameters(:, 2) = [0    0 0 0 1.0  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 ];
parameters(:, 3) = [-0.5 0 0 0 1.5  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 ];
parameters(:, 4) = [-1.0 0 0 0 2.0  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 ];
parameters(:, 5) = [0.5  0 0 0 0.5  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 ];
parameters(:, 6) = [1.0  0 0 0 0.0  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 ];
parameters(:, 7) = [1.5  0 0 0 -0.5 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 ];
parameters(:, 8) = [2.0  0 0 0 -1.0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 ];

for i1 = 1 : size(parameters, 2);
    bvhdata_2{i1} = f_extrapolation_nway(parameters(:, i1), walk_motions_a);
    coordinates2{i1} = f_fast_bvh_to_coordinates(bvhdata_2{i1}(1:end, :), walk_motions_a(1).skel);
end
    
% Visualization
for i1 = 1 : length(bvhdata_2);
    f_animate(bvhdata_2{i1}, walk_motions_a(1).skel, coordinates2{i1}, 'figNum', 200+i1);
    title(['Extrapolation: motion A ' num2str(parameters(1, i1) * 100) '%, motion B ' num2str(parameters(5, i1)*100) '%']);
    pause(1.0);
end
