clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');

%% Description of the example

% This example demonstrates synthesis of new motions by scaling
% frequency bands. Motions two, three and four are cases where the bands
% are scaled equally (except the lowest 'constant' band). Motion five,
% six and seven are cases where the amplification of the bands varies.
% Unequal scaling can cause visible foot slide that cannot be removed by
% adjusting movement of the root joint as in the cases of equally scaled
% bands. Other visible artifacts caused by too large scaling of a band
% include visible high frequency noise and joints going outside their
% usual limits (for example knees bending the backwards).

% Equal scaling of bands seems to directly affect the size of motions (for
% example step size). The equal scaling seems to also affect how tense or
% relaxed the motion appears. The effects of unequal scaling on style are
% more difficult to describe.   

%% Frequency band scaling

load('matlab_data/walks_a_warped.mat'); % Run script_1 to create this file
m = walk_motions_a(1);

% Uncomment following lines to apply frequency band scaling to a hand waving motion
% load('matlab_data/waves_1.mat');
% m = wave_motions_a(1);

clear parameters;
parameters(1, :) = [1 1 1 1 1 1 1];
parameters(2, :) = [2 2 2 2 2 2 2];
parameters(3, :) = [0.2 0.2 0.2 0.2 0.2 0.2 0.2];
parameters(4, :) = [-0.5 -0.5 -0.5 -0.5 -0.5 -0.5 -0.5];
parameters(5, :) = [6 6 6 1 1 1 1];
parameters(6, :) = [1 1 5 5 1 1 1];
parameters(7, :) = [1 1 1 -1 -1 1 1];


for i1 = 1 : size(parameters, 1)
    bvhdata{i1} = f_frequency_modification(m.bvhdata, m.quaternions, parameters(i1, :));
    temp_coordinates = f_fast_bvh_to_coordinates(bvhdata{i1}, m.skel);
    bvhdata{i1} = f_fix_foot_slide_for_locomotion(bvhdata{i1}, m.coordinates, temp_coordinates);
    coordinates{i1} = f_fast_bvh_to_coordinates(bvhdata{i1}, m.skel);
end

%% Visualization

for i1 = 1 : length(coordinates)
    f_animate(bvhdata{i1}, m.skel, coordinates{i1}, 'figNum', 101);
    if (std(parameters(i1, :)) < 0.00001)
        title(['Frequency bands equally scaled by factor of ' num2str(parameters(i1, 1))]);
    else
        title('Frequency bands unequally scaled');
    end
    pause(1.0);
end