clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');

%% Description of the example

% This example demonstrates synthesis of new motions by swapping frequency
% bands between two motions. In the first three cases, only the lowest
% "constant" frequency band has been taken from another motion. The last
% case has three bands replaced. Swapping only the lowest band seems to
% create new style variations with a new overall posture and without much
% artifacts, while swapping other bands can result in foot slide and
% abnormal accelerations.

%%

load('matlab_data/walks_a_warped.mat'); % Run script_1 to create this file

%% Swapping frequency bands of quaternion signals

clear channels_to_swap new_motions new_coordinates;

motion_pairs = [
    1 18;
    1 18;
    5 6;
    5 6;
    6 5;
    6 5;
    6 5
    ];

channels_to_swap{1} = [];
channels_to_swap{2} = 8;
channels_to_swap{3} = [];
channels_to_swap{4} = 8;
channels_to_swap{5} = [];
channels_to_swap{6} = 8;
channels_to_swap{7} = 3:6;

for i1 = 1 : length(channels_to_swap)
    
    new_motions{i1} = walk_motions_a(motion_pairs(i1, 1));
    
    bands_1 = f_multiresolution_split(walk_motions_a(motion_pairs(i1, 1)).warped_quaternions);
    bands_2 = f_multiresolution_split(walk_motions_a(motion_pairs(i1, 2)).warped_quaternions);
    bands_1(:, :, channels_to_swap{i1}) = bands_2(:, :, channels_to_swap{i1});
    new_motions{i1}.quaternions = sum(bands_1, 3);
    new_motions{i1}.bvhdata = new_motions{i1}.warped_bvh;
    time_transfer_inverted = interp1(new_motions{i1}.time_transfer, 1:size(new_motions{i1}.bvhdata, 1), 1:max(new_motions{i1}.time_transfer));
    new_motions{i1}.bvhdata = f_warp_motion(time_transfer_inverted, new_motions{i1});
    new_coordinates{i1} = f_fast_bvh_to_coordinates(new_motions{i1}.bvhdata, walk_motions_a(1).skel);
end

%% Visualization

for i1 = 1:length(new_coordinates)
    f_animate(new_motions{i1}.bvhdata, walk_motions_a(1).skel, new_coordinates{i1}, 'figNum', 100);
    if (isempty(channels_to_swap{i1}))
        title('Original motion')
    else
        if (channels_to_swap{i1}(1) == 8)
            title('Motion with swapped lowest frequency band')
        else
            title('Motion with partially mixed frequency bands');
        end
    end
    pause(0.5);
end