clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');

%% Description of the example

% This example demonstrates creation of new motions by swapping rotations
% of individual joints or groups of joints. In this example, the rotations
% are not broken up in components. Swapping rotations can produce new style
% variations with almost no artifacts in cases where the joints are
% selected carefully (for example swapping a whole arm or the whole upper
% body). However, swapping joints that are part of limbs that have contacts
% with rigid objects can easily break the contacts. Also, partially
% swapping the joints in a limb can make the motion of the end of the
% limb shaky.

%%

load('matlab_data/walks_a_warped.mat'); % Run script_1 to create this file

body_parts_names = {walk_motions_a(1).skel.tree.name}';
body_parts_names(strcmp(body_parts_names(:), 'Site')) = [];

%% Swapping rotations of joints

motion_pairs = [
    5 1;
    1 5;
    5 1;
    1 5;
    5 1;
    5 1;
    5 1;
];

titles = {
    'Original motion'
    'Original motion'
    'Swapped upper body'
    'Swapped upper body'
    'Swapped right arm'
    'Swapped spine (from abs to head)'
    'Random joints swapped'
    };

clear channels_to_swap;
channels_to_swap{1} = [];
channels_to_swap{2} = [];
channels_to_swap{3} = (6*3) + 1 : (13*3) + 3;
channels_to_swap{4} = (6*3) + 1 : (13*3) + 3;
channels_to_swap{5} = ((6*3) + 1 : (9*3) + 3);
channels_to_swap{6} = ((1*3) + 1 : (5*3) + 3);
          
channels_to_swap{7} = rand(1, 60) < 0.5;
channels_to_swap{7}(2:3:60) = channels_to_swap{7}(1:3:60);
channels_to_swap{7}(3:3:60) = channels_to_swap{7}(1:3:60);

%% Visualization

for i1 = 1 : length(channels_to_swap)
    new_bvhdata{i1} = f_swap_body_parts_prewarped( channels_to_swap{i1}, walk_motions_a(motion_pairs(i1, 1)), walk_motions_a(motion_pairs(i1, 2)));
    new_coordinates{i1} = f_fast_bvh_to_coordinates(new_bvhdata{i1}, walk_motions_a(1).skel);
    f_animate(new_bvhdata{i1}, walk_motions_a(1).skel, new_coordinates{i1}, 'figNum', 100 +i1);
    title(titles{i1}); 
    pause(1);
end