clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');

%% Description of the example

% This example demonstrates effects of swapping individual quaternion
% channels between motions. The cases where the same quaternion component
% is swapped in each rotation show that quaternion components as such do
% not have a straight forward mapping to perceived styles. Also, the cases
% where one component has been swapped, look more shaky and contain more
% sudden accelerations than the original motion or the motion where all
% components have been swapped. A possible reason for the behaviour can be
% that after swapping components the quaternions do not generally sum to
% one. Thus, small changes in the component values may be magnified as the
% quaternions are normalized to unit length.

% The point of this example is not to present a sensible motion
% modification, but rather to demonstrate the same artifacts, in a more
% simple case, that the PCA (ex_13) and ICA (ex_14) swapping can produce.

%%

load('matlab_data/walks_a_warped.mat'); % Run script_1 to create this file

%% Swapping signals of quaternion channels

clear channels_to_swap new_motions new_coordinates;

motion_pairs = [
    1 5;
    1 5;
    1 5;
    1 5;
    1 5;
    1 5;
    ];

channels_to_swap{1} = [];
channels_to_swap{2} = 1:4:76;
channels_to_swap{3} = 2:4:76;
channels_to_swap{4} = 3:4:76;
channels_to_swap{5} = 4:4:76;
channels_to_swap{6} = 1:76;

%% Visualization

titles = {
    'Original motion'
    'Quaternion X channels swapped'
    'Quaternion Y channels swapped'
    'Quaternion Z channels swapped'
    'Quaternion W channels swapped'
    'Swapped all rotations'
    };

for i1 = 1 : length(channels_to_swap)
    new_bvhdata{i1} = f_swap_channels_prewarped( channels_to_swap{i1}, walk_motions_a(motion_pairs(i1, 1)), walk_motions_a(motion_pairs(i1, 2)));
    new_coordinates{i1} = f_fast_bvh_to_coordinates(new_bvhdata{i1}, walk_motions_a(1).skel);
    f_animate(new_bvhdata{i1}, walk_motions_a(1).skel, new_coordinates{i1}, 'figNum', 100 +i1);
    title(titles{i1}); 
    pause(1);
end
