function [ final_bvhdata ] = f_extrapolation_pairwise( m_from, m_to, percent)
%F_EXTRAPOLATION_PAIRWISE Pairwise extrapolation between motions
%
%   F_EXTRAPOLATION_PAIRWISE(M_FROM,M_TO,PERCENT)
%   M_FROM and M_TO are the two motions.
%   PERCENT tells the extrapolation amount. Values 0 and 1 correspond to
%   the original motions. Good extrapolations are often in range between 
%   -1 to 2.
%   Returns BVH data for the new motion.

% Copyright (c) 2015, Klaus Förger

bvh_size = size(m_from.bvhdata, 2);
rot_dof = (bvh_size/3) - 1;

m_new = [];

% Warping second motion to same length as the first one.
time_transfer = 1:(size(m_to.bvhdata, 1) - 1) / (size(m_from.bvhdata, 1)-1):size(m_to.bvhdata, 1);
m_to_bvh_warped = f_warp_motion(time_transfer, m_to);
m_to_quat_warped = f_calculate_quaternions(m_to_bvh_warped);

% Extrapolation

m_new.quaternions = zeros(size(m_from.quaternions));

for i1 = 1 : rot_dof
    for i2 = 1 : size(m_new.quaternions, 1)
        s = 1+(4*(i1-1)) : 4+(4*(i1-1));
        m_new.quaternions(i2, s) = f_slerp(m_from.quaternions(i2, s), m_to_quat_warped(i2, s), percent, 0.001);
    end
end

m_new.bvhdata = zeros(size(m_from.bvhdata));
m_new.bvhdata(:, 1:3) = (m_from.bvhdata(:, 1:3) * (1-percent)) +  (m_to_bvh_warped(:, 1:3) * percent);

% Time warping the resulting motion to the duration the reflects the
% extrapolation parameter.
target_length = round(exp((log(size(m_from.bvhdata, 1)) * (1-percent) + (log(size(m_to.bvhdata, 1)) * percent))));
time_transfer_back = 1:(size(m_from.bvhdata, 1) - 1) / (target_length-1):size(m_from.bvhdata, 1);

final_bvhdata = f_warp_motion(time_transfer_back, m_new);


end
