function xyz = f_fast_bvh_to_coordinates(bvhdata, skel)
%F_FAST_BVH_TO_COORDINATES(BVHDATA, SKEL) Returns joint coordinates of a bvh motion
%   This function is an optimized version of the f_bvh_to_coordinates.

% Copyright (c) 2015, Klaus Förger

xyz = zeros(length(skel.tree), 3, size(bvhdata, 1));
parents = [skel.tree(:).parent];
offsets = cell2mat({skel.tree(:).offset}');
rotInd = zeros(length(skel.tree), 3);
for i1 = 1 : length(skel.tree)
    if ~isempty(skel.tree(i1).rotInd)
        rotInd(i1, :) = skel.tree(i1).rotInd;
    end
end
bvhdata(:, 4:end) = bvhdata(:, 4:end)./180.*pi;


all_rotations = zeros(3, 3, length(skel.tree), size(bvhdata, 1));
all_rotations(1, 1, :, :) = 1;
all_rotations(2, 2, :, :) = 1;
all_rotations(3, 3, :, :) = 1;
sin_chan = sin(bvhdata);
cos_chan = cos(bvhdata);


for i2 = 1 : length(skel.tree)
    if ~isempty(skel.tree(i2).children)
        ind_1 = rotInd(i2, 1);
        ind_2 = rotInd(i2, 2);
        ind_3 = rotInd(i2, 3);
        for i1 = 1 : size(bvhdata, 1)
            c1 = cos_chan(i1, ind_1);
            c2 = cos_chan(i1, ind_2);
            c3 = cos_chan(i1, ind_3);
            s1 = sin_chan(i1, ind_1);
            s2 = sin_chan(i1, ind_2);
            s3 = sin_chan(i1, ind_3);
            all_rotations(:, :, i2, i1) = [c2*c3-s1*s2*s3 c2*s3+s1*s2*c3 -s2*c1
                -c1*s3 c1*c3 s1
                s2*c3+c2*s1*s3 s2*s3-c2*s1*c3 c2*c1];
        end
    end
end


hierarchical_rot = zeros(3, 3, length(skel.tree), size(bvhdata, 1));

hierarchical_rot(:, :, 1, :) = all_rotations(:, :, 1, :);


for i1 = 1 : size(bvhdata, 1)
    xyz(1, :, i1) = bvhdata(i1, [1 2 3]);
end


for i = 2:length(skel.tree)
    
    this_par = parents(i);
    this_offset = offsets(i, :);
    
    for i1 = 1 : size(bvhdata, 1)
         
        thisRotation = all_rotations(:, :, i, i1);
        
        xyz(i, :, i1) = this_offset * hierarchical_rot(:, :, this_par, i1) + xyz(this_par, :, i1);
        hierarchical_rot(:, :, i, i1) = thisRotation * hierarchical_rot(:, :, this_par, i1);
        
    end
end

xyz = reshape(permute(xyz, [3 2 1]), size(xyz, 3), length(skel.tree) * 3);


