function [ output_bvhdata ] = f_fix_foot_slide_for_locomotion( bvhdata, old_coordinates, new_coordinates )
%F_FIX_FOOT_SLIDE_FOR_LOCOMOTION Tries to fix foot slide for locomotion sequences
%
%   F_FIX_FOOT_SLIDE_FOR_LOCOMOTION(BVHDATA,OLD_COORDINATES,NEW_COORDINATES)
%   Takes the bvhdata of a locomotion sequence after frequency based
%   editing, and attempts to fix foot sliding by rescaling the coordinates
%   of the root joint.
%
%   Returns BVH data for the fixed motion.
%
%   Warning: This does not work with other actions than locomotion, and
%   currently the code assumes the same skeleton structure as the example
%   motions included in the toolbox. Therefore, this code does not
%   represent a generic solution of fixing foot slide.

% Copyright (c) 2015, Klaus Förger

overall_translation = norm((bvhdata(1, 1:3) - bvhdata(end, 1:3)));

if (overall_translation > 30) % Checking if character is doing any locomotion
    
    output_bvhdata = bvhdata;
    
    % Removing effect of the root translation
    new_coordinates = new_coordinates - repmat(new_coordinates(:, 1:3), 1, 24);
    
    % A and B are the two legs
    
    old_coords_A = old_coordinates(:, [58 59 60]);
    old_coords_B = old_coordinates(:, [70 71 72]);
    new_coords_A = new_coordinates(:, [58 59 60]);
    new_coords_B = new_coordinates(:, [70 71 72]);
    
    old_vel_A = old_coords_A(1:end-1, :) - old_coords_A(2:end, :);
    old_vel_B = old_coords_B(1:end-1, :) - old_coords_B(2:end, :);
    new_vel_A = new_coords_A(1:end-1, :) - new_coords_A(2:end, :);
    new_vel_B = new_coords_B(1:end-1, :) - new_coords_B(2:end, :);
    
    stopped_frames_A = medfilt1(double((sum(abs(old_vel_A), 2) < 0.15)), 15); % 0.15 is a magic number, it could break on some motions
    stopped_frames_B = medfilt1(double((sum(abs(old_vel_B), 2) < 0.15)), 15); % 0.15 is a magic number, it could break on some motions
    
    root_movement = old_coordinates(:, 1:3) - repmat(old_coordinates(1, 1:3), size(old_coordinates, 1), 1);
    root_vel = root_movement(1:end-1, :) - root_movement(2:end, :);
    
    error = [];
    index = 0;
    for i1 = -3:0.02:3
        index = index + 1;
        test_vel_A = new_vel_A(logical(stopped_frames_A),1:3) + (root_vel(logical(stopped_frames_A), 1:3)*i1);
        test_vel_B = new_vel_B(logical(stopped_frames_B),1:3) + (root_vel(logical(stopped_frames_B), 1:3)*i1);
        error_A = sum(mean(abs(test_vel_A - old_vel_A(logical(stopped_frames_A),1:3))));
        error_B = sum(mean(abs(test_vel_B - old_vel_B(logical(stopped_frames_B),1:3))));
        error(index, 1) = error_A + error_B;
        error(index, 2) = i1;
    end
    
    error = sortrows(error, 1);
    
    ratio = error(1, 2);
    
    output_bvhdata(:, 1:3) = repmat(old_coordinates(1, 1:3), size(old_coordinates, 1), 1) + (root_movement * ratio);
    
end

end

