function [ output_bvhdata ] = f_frequency_modification( bvhdata, quat, parameters, extend )
%F_FREQUENCY_MODIFICATION Performs frequency band scaling for a motion.
%
%   F_FREQUENCY_MODIFICATION(BVHDATA,QUATERNIONS,PARAMETERS)
%   The parameters are an array of seven numbers that represent the multipliers
%   of each frequency band. An array of [1 1 1 1 1 1 1] keeps the
%   bands unscaled. Good parameter values are between -2 and 2.
%   Returns BVH data for the new motion.
%
%   F_FREQUENCY_MODIFICATION(BVHDATA,QUATERNIONS,PARAMETERS,EXTEND)
%   Giving 'extend = true' extends the motion from the start and end by 100
%   frames. This can be useful if the original motion starts or ends during
%   a fast movement.

% Copyright (c) 2015, Klaus Förger

bvh_size = size(bvhdata, 2);
rot_dof = (bvh_size/3) - 1;

if nargin < 4
    extend = false;
end

quat_new = f_fix_quaternions(quat);

%% Extending data

if (extend)
    bvhdata = [repmat(bvhdata(1, :), 100, 1); bvhdata; repmat(bvhdata(end, :), 100, 1)];
    quat_new = [repmat(quat_new(1, :), 100, 1); quat_new; repmat(quat_new(end, :), 100, 1)];
end

%% Multiresolution editing

quat_new = f_multiresolution_editing(quat_new, parameters);

%% Returning to bvh format

output_bvhdata = zeros(size(bvhdata));

for i2 = 1 : rot_dof
    output_bvhdata(:, (i2-1)*3+4:(i2-1)*3+6) = f_SpinCalc('QtoEA312', quat_new(:, (i2-1)*4+1:(i2-1)*4+4), 0.0001, 0);
end

output_bvhdata(:, 1:3) = bvhdata(:, 1:3);

end

