function [ final_bvhdata ] = f_parametric_component_synthesis( scores, coeffs_to_synth, mean_values, start_motion )
%F_PARAMETRIC_COMPONENT_SYNTHESIS(SCORES,COEFFS_TO_SYNTH,MEAN_VALUES,START_MOTION)
%   Performs motion synthesis based on a given decomposition and parameters.
%   SCORES are scores from PCA or ICA decomposition.
%   COEFFS_TO_SYNTH are the coefficients for the synthesis.
%   MEAN_VALUES are the mean values that were subtracted to make the data
%   have a zero mean.
%   START_MOTION is a motion structure with required skeleton data.
%   Returns BVH data for the new motion.


% Copyright (c) 2015, Klaus Förger

bvh_size = size(start_motion.bvhdata, 2);
rot_dof = (bvh_size/3) - 1;
num_of_quaternion_channels = size(start_motion.quaternions, 2);
frames_after_timewarp = size(start_motion.warped_bvh, 1);

synthesized_stuff = scores * coeffs_to_synth;
synthesized_stuff = synthesized_stuff + mean_values';
synthesized_stuff = reshape(synthesized_stuff, frames_after_timewarp, num_of_quaternion_channels+4);
bvh_new = zeros(size(start_motion.warped_bvh));
for i2 = 1 : rot_dof
    bvh_new(:, (i2-1)*3+4:(i2-1)*3+6) = f_SpinCalc('QtoEA312', synthesized_stuff(:, (i2-1)*4+1+4:(i2-1)*4+4+4), 0.0001, 0);
end
bvh_new(:, 1:3) = synthesized_stuff(:, 1:3);

clear motion;
motion(1).skel = start_motion.skel;
motion(1).bvhdata = bvh_new;

motion(1).quaternions = f_calculate_quaternions(bvh_new);

% Note: The following inverse time warping is only approximate as
% extrapolating the warp can cause visually unpleasing distortions
trans_offset =  synthesized_stuff(:, 4);

trans_offset = cumsum(exp(trans_offset));
trans_offset(1) = 1;

trans_inverted = interp1(trans_offset, 1:size(bvh_new, 1), 1:max(trans_offset));
trans_inverted(1) = 1;

final_bvhdata = f_warp_motion(trans_inverted, motion(1));


end

