function [ shown_frames_count ] = f_render_skeleton_pair(bvhdata1, bvhdata2, skel1, skel2, clear_in_the_end, color, save_frames, fps, skeleton_quality, bone_radius, light_pos, video_writer)
%F_RENDER_SKELETON_PAIR A function used in the animation loop

if nargin < 7
    save_frames = false;
end

if nargin < 8
    fps = 100;
end

if nargin < 9
    skeleton_quality = 1;
end

if nargin < 10
    bone_radius = 2;
end

if nargin < 11
    light_pos = [-1000 0 1000];
end

if nargin < 12
    video_writer = [];
end

handle1 = [];
handle2 = [];

time_end_1 = size(bvhdata1, 1) / fps;
time_end_2 = size(bvhdata2, 1) / fps;

light_count = 0;

% Creating bones and transformation hierarchy 1
for i1 = 1 : length(skel1.tree) 
    new_transform = hgtransform();
    
    if (isempty(skel1.tree(i1).children) && light_count < 8)
        % Matlab seems want the light to be set for each end of limb
        light_count = light_count + 1;
        light('Position', light_pos, 'Style', 'infinite');
    end
    
    if (skel1.tree(i1).parent > 0)
        
        if (skeleton_quality > 1 || save_frames)
            % This enables higher quality for videos
            bone_visuals = f_create_rotation_bone(skel1.tree(i1).offset, bone_radius, 12, color);
        else
            bone_visuals = f_create_rotation_bone(skel1.tree(i1).offset, bone_radius, 6, color);
        end
        
        set(bone_visuals, 'Parent', handle1(skel1.tree(i1).parent));
        set(new_transform, 'Parent', handle1(skel1.tree(i1).parent));
    end
    handle1(end+1) = new_transform;
end

% Creating bones and transformation hierarchy 2
for i1 = 1 : length(skel2.tree) 
    new_transform = hgtransform();
    
    if (skel2.tree(i1).parent > 0)
        
        if (skeleton_quality > 1 || save_frames)
            % This enables higher quality for videos
            bone_visuals = f_create_rotation_bone(skel2.tree(i1).offset, bone_radius, 12, color);
        else
            bone_visuals = f_create_rotation_bone(skel2.tree(i1).offset, bone_radius, 6, color);
        end
        
        set(bone_visuals, 'Parent', handle2(skel2.tree(i1).parent));
        set(new_transform, 'Parent', handle2(skel2.tree(i1).parent));
    end

    handle2(end+1) = new_transform;
end

% Precalculating hg transforms
disp('Precalculating visualization...');
ident = [1 0 0 0; 0 1 0 0; 0 0 1 0; 0 0 0 1];
hg1 = zeros(4, 4, size(bvhdata1, 1), length(handle1));
hg2 = zeros(4, 4, size(bvhdata2, 1), length(handle2));
for i1 = 1 : size(bvhdata1, 1)
    for i2 = 1 : length(handle1)

        % Calculating rotation matrices
        if (isempty(skel1.tree(i2).rotInd))
            hg1(:, :, i1, i2) = ident;
        else
            R1 = ident;
            R1(1, 1) = cos(-bvhdata1(i1, skel1.tree(i2).rotInd(3)) * (pi/180));
            R1(3, 3) = R1(1, 1);
            R1(1, 3) = sin(-bvhdata1(i1, skel1.tree(i2).rotInd(3)) * (pi/180));
            R1(3, 1) = -R1(1, 3);
            
            R2 = ident;
            R2(2, 2) = cos(-bvhdata1(i1, skel1.tree(i2).rotInd(1)) * (pi/180));
            R2(3, 3) = R2(2, 2);
            R2(2, 3) = -sin(-bvhdata1(i1, skel1.tree(i2).rotInd(1)) * (pi/180));
            R2(3, 2) = -R2(2, 3);
            
            R3 = ident;
            R3(1, 1) = cos(-bvhdata1(i1, skel1.tree(i2).rotInd(2)) * (pi/180));
            R3(2, 2) = R3(1, 1);
            R3(1, 2) = -sin(-bvhdata1(i1, skel1.tree(i2).rotInd(2)) * (pi/180));
            R3(2, 1) = -R3(1, 2);
            
            hg1(:, :, i1, i2) = R1 * R2 * R3;
        end
        
        % Adding translation to the matrix
        if (isempty(skel1.tree(i2).posInd))
            hg1(1:3, 4, i1, i2) = skel1.tree(i2).offset([1 3 2]);
        else
            hg1(1:3, 4, i1, i2) = bvhdata1(i1,skel1.tree(i2).posInd([1 3 2]));
        end
        
    end
end
for i1 = 1 : size(bvhdata2, 1)
    for i2 = 1 : length(handle2)

        % Calculating rotation matrices
        if (isempty(skel2.tree(i2).rotInd))
            hg2(:, :, i1, i2) = ident;
        else
            R1 = ident;
            R1(1, 1) = cos(-bvhdata2(i1, skel2.tree(i2).rotInd(3)) * (pi/180));
            R1(3, 3) = R1(1, 1);
            R1(1, 3) = sin(-bvhdata2(i1, skel2.tree(i2).rotInd(3)) * (pi/180));
            R1(3, 1) = -R1(1, 3);
            
            R2 = ident;
            R2(2, 2) = cos(-bvhdata2(i1, skel2.tree(i2).rotInd(1)) * (pi/180));
            R2(3, 3) = R2(2, 2);
            R2(2, 3) = -sin(-bvhdata2(i1, skel2.tree(i2).rotInd(1)) * (pi/180));
            R2(3, 2) = -R2(2, 3);
            
            R3 = ident;
            R3(1, 1) = cos(-bvhdata2(i1, skel2.tree(i2).rotInd(2)) * (pi/180));
            R3(2, 2) = R3(1, 1);
            R3(1, 2) = -sin(-bvhdata2(i1, skel2.tree(i2).rotInd(2)) * (pi/180));
            R3(2, 1) = -R3(1, 2);
            
            hg2(:, :, i1, i2) = R1 * R2 * R3;
        end
        
        % Adding translation to the matrix
        if (isempty(skel1.tree(i2).posInd))
            hg2(1:3, 4, i1, i2) = skel2.tree(i2).offset([1 3 2]);
        else
            hg2(1:3, 4, i1, i2) = bvhdata2(i1,skel2.tree(i2).posInd([1 3 2]));
        end
        
    end
end
disp('Done.')

shown_frames_count = 0;

if (save_frames || ~isempty(video_writer))
    for frame_number = 1 : max(size(bvhdata1, 1), size(bvhdata2, 1));
        shown_frames_count = shown_frames_count + 1;
        
        % Updating the hg transformations
        for i2 = 1 : length(handle1)
            set(handle1(i2),'Matrix',hg1(:, :,  min(frame_number, size(bvhdata1, 1)), i2));
        end
        for i2 = 1 : length(handle2)
            set(handle2(i2),'Matrix',hg2(:, :,  min(frame_number, size(bvhdata2, 1)), i2));
        end

        drawnow;

        if (shown_frames_count == 1)
            pause(0.5); % This is usually enought to avoid broken first frames
            actual_figure_pos = get(gcf, 'Position');
        end
        if (~isempty(video_writer))
            frame = getframe(gcf);
            writeVideo(video_writer, frame);
        end
        if (save_frames)
            screencapture(0, actual_figure_pos, 'target', ['video_frames/frame_' sprintf('%05d',  shown_frames_count) '.png']);
        end
    end
    shown_frames_count = frame_number;
    
    
else % Realtime animation, not saving frames to disk
    
    tic;
    prev_frame = 0;
    frame_number = 1;
    last_frame_not_shown = true;
    
    while (last_frame_not_shown)
        
        if (frame_number == max(size(bvhdata1, 1), size(bvhdata2, 1)))
            last_frame_not_shown = false;
        end
        
        if (frame_number > prev_frame)
            
            % Updating the hg transformations
            for i2 = 1 : length(handle1)
                set(handle1(i2),'Matrix',hg1(:, :,  min(frame_number, size(bvhdata1, 1)), i2));
            end
            for i2 = 1 : length(handle2)
                set(handle2(i2),'Matrix',hg2(:, :,  min(frame_number, size(bvhdata2, 1)), i2));
            end

            shown_frames_count = shown_frames_count + 1;
            drawnow;
        end
        
        time_next = (frame_number + 1) * (1/fps);
        
        time_now = toc;
        if (time_now < time_next)
            pause(min((time_next - time_now), 0.001));
        end
        
        prev_frame = frame_number;
        time_now = toc;
        frame_number = min(ceil(time_now * fps), max(size(bvhdata1, 1), size(bvhdata2, 1)));

    end
    
end

if (clear_in_the_end)
    while (~isempty(handle1))
        delete(handle1(end));
        handle1(end) = [];
    end
    while (~isempty(handle2))
        delete(handle2(end));
        handle2(end) = [];
    end
end

end