function [ bvh_new ] = f_style_transfer( m_from, m_to, m_start, percent)
%F_STYLE_TRANSFER Performs a style transfer
%
%   F_STYLE_TRANSFER(M_FROM,M_TO,M_START,PERCENT)
%   M_FROM and M_TO are two motions that define a style difference.
%   M_START is the motion where the difference is transferred to.
%   PERCENT tells the multiplier for the difference. Good values are often
%   in range between 0 to 2. Zero means that no style is transferred.
%   Returns BVH data for the new motion.

% Copyright (c) 2015, Klaus Förger

bvh_size = size(m_start.bvhdata, 2);
rot_dof = (bvh_size/3) - 1;

t_from = [0; log(((m_from.time_transfer(2:end) - m_from.time_transfer(1:end-1)))')];
t_to = [0; log(((m_to.time_transfer(2:end) - m_to.time_transfer(1:end-1)))')];
t_start = [0; log(((m_start.time_transfer(2:end) - m_start.time_transfer(1:end-1)))')];

t_new = t_start + ((t_to - t_from) * percent);

trans_temp = cumsum(exp(t_new))';

time_tranfer = interp1(trans_temp, 1:size(m_start.warped_bvh, 1), 1:max(trans_temp));
time_tranfer(1) = 1;

length_from = size(m_from.warped_bvh, 1);
length_to = size(m_to.warped_bvh, 1);
length_start = size(m_start.warped_bvh, 1);

% Timewarp of rotations
for i1 = 1 : rot_dof*4
    quat_from(:, i1) = interp1(1:length_from, m_from.warped_quaternions(:, i1), time_tranfer);
    quat_to(:, i1) = interp1(1:length_to, m_to.warped_quaternions(:, i1), time_tranfer);
    quat_start(:, i1) = interp1(1:length_to, m_start.warped_quaternions(:, i1), time_tranfer);
end

% Linear extrapolation of quaternion offsets
quat_extrapolated_offsets = (quat_to - quat_from) * percent;
quat_tranferred = quat_start + quat_extrapolated_offsets;

% Timewarp of root coordinates
for i1 = 1 : 3
    root_from(:, i1) = interp1(1:length_from, m_from.warped_bvh(:, i1), time_tranfer);
    root_to(:, i1) = interp1(1:length_to, m_to.warped_bvh(:, i1), time_tranfer);
    root_start(:, i1) = interp1(1:length_start, m_start.warped_bvh(:, i1), time_tranfer);
end

% Linear extrapolation of root coordinate offsets
root_extrapolated_offsets = (root_to - root_from) * percent;
root_tranferred = root_start + root_extrapolated_offsets;

% Returning all to BVH -format

for i2 = 1 : rot_dof
    bvh_new(:, (i2-1)*3+4:(i2-1)*3+6) = f_SpinCalc('QtoEA312', quat_tranferred(:, (i2-1)*4+1:(i2-1)*4+4), 0.0001, 0);
end

bvh_new(:, 1:3) = root_tranferred;

end
