clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');

%%

% This script loads a collection of 108 hand waving motions. The collection
% is then saved to disk to be readily used by other examples.

%% Load originals

files = {
    'motion_data/wave_1_p1.bvh'
    'motion_data/wave_2_p1.bvh'
    'motion_data/wave_3_p1.bvh'
    'motion_data/wave_4_p1.bvh'
    'motion_data/wave_5_p1.bvh'
    'motion_data/wave_6_p1.bvh'
    'motion_data/wave_7_p1.bvh'
    'motion_data/wave_8_p1.bvh'
    'motion_data/wave_9_p1.bvh'
    'motion_data/wave_10_p1.bvh'
    'motion_data/wave_11_p1.bvh'
    'motion_data/wave_12_p1.bvh'
    'motion_data/wave_13_p1.bvh'
    'motion_data/wave_14_p1.bvh'
    'motion_data/wave_15_p1.bvh'
    'motion_data/wave_16_p1.bvh'
    'motion_data/wave_17_p1.bvh'
    'motion_data/wave_18_p1.bvh'
    'motion_data/wave_19_p1.bvh'
    'motion_data/wave_20_p1.bvh'
    'motion_data/wave_21_p1.bvh'
    'motion_data/wave_22_p1.bvh'
    'motion_data/wave_23_p1.bvh'
    'motion_data/wave_24_p1.bvh'
    'motion_data/wave_25_p1.bvh'
    'motion_data/wave_26_p1.bvh'
    'motion_data/wave_27_p1.bvh'
    'motion_data/wave_28_p1.bvh'
    'motion_data/wave_29_p1.bvh'
    'motion_data/wave_30_p1.bvh'
    'motion_data/wave_31_p1.bvh'
    'motion_data/wave_32_p1.bvh'
    'motion_data/wave_33_p1.bvh'
    'motion_data/wave_34_p1.bvh'
    'motion_data/wave_35_p1.bvh'
    'motion_data/wave_36_p1.bvh'
    'motion_data/wave_37_p1.bvh'
    'motion_data/wave_38_p1.bvh'
    'motion_data/wave_39_p1.bvh'
    'motion_data/wave_40_p1.bvh'
    'motion_data/wave_41_p1.bvh'
    'motion_data/wave_42_p1.bvh'
    'motion_data/wave_43_p1.bvh'
    'motion_data/wave_44_p1.bvh'
    'motion_data/wave_45_p1.bvh'
    'motion_data/wave_46_p1.bvh'
    'motion_data/wave_47_p1.bvh'
    'motion_data/wave_48_p1.bvh'
    'motion_data/wave_49_p1.bvh'
    'motion_data/wave_50_p1.bvh'
    'motion_data/wave_51_p1.bvh'
    'motion_data/wave_52_p1.bvh'
    'motion_data/wave_53_p1.bvh'
    'motion_data/wave_54_p1.bvh'
    'motion_data/wave_55_p1.bvh'
    'motion_data/wave_56_p1.bvh'
    'motion_data/wave_57_p1.bvh'
    'motion_data/wave_58_p1.bvh'
    'motion_data/wave_59_p1.bvh'
    'motion_data/wave_60_p1.bvh'
    'motion_data/wave_61_p1.bvh'
    'motion_data/wave_62_p1.bvh'
    'motion_data/wave_63_p1.bvh'
    'motion_data/wave_64_p1.bvh'
    'motion_data/wave_65_p1.bvh'
    'motion_data/wave_66_p1.bvh'
    'motion_data/wave_67_p1.bvh'
    'motion_data/wave_68_p1.bvh'
    'motion_data/wave_69_p1.bvh'
    'motion_data/wave_70_p1.bvh'
    'motion_data/wave_71_p1.bvh'
    'motion_data/wave_72_p1.bvh'
    'motion_data/wave_73_p1.bvh'
    'motion_data/wave_74_p1.bvh'
    'motion_data/wave_75_p1.bvh'
    'motion_data/wave_76_p1.bvh'
    'motion_data/wave_77_p1.bvh'
    'motion_data/wave_78_p1.bvh'
    'motion_data/wave_79_p1.bvh'
    'motion_data/wave_80_p1.bvh'
    'motion_data/wave_81_p1.bvh'
    'motion_data/wave_82_p1.bvh'
    'motion_data/wave_83_p1.bvh'
    'motion_data/wave_84_p1.bvh'
    'motion_data/wave_85_p1.bvh'
    'motion_data/wave_86_p1.bvh'
    'motion_data/wave_87_p1.bvh'
    'motion_data/wave_88_p1.bvh'
    'motion_data/wave_89_p1.bvh'
    'motion_data/wave_90_p1.bvh'
    'motion_data/wave_91_p1.bvh'
    'motion_data/wave_92_p1.bvh'
    'motion_data/wave_93_p1.bvh'
    'motion_data/wave_94_p1.bvh'
    'motion_data/wave_95_p1.bvh'
    'motion_data/wave_96_p1.bvh'
    'motion_data/wave_97_p1.bvh'
    'motion_data/wave_98_p1.bvh'
    'motion_data/wave_99_p1.bvh'
    'motion_data/wave_100_p1.bvh'
    'motion_data/wave_101_p1.bvh'
    'motion_data/wave_102_p1.bvh'
    'motion_data/wave_103_p1.bvh'
    'motion_data/wave_104_p1.bvh'
    'motion_data/wave_105_p1.bvh'
    'motion_data/wave_106_p1.bvh'
    'motion_data/wave_107_p1.bvh'
    'motion_data/wave_108_p1.bvh'
    };

for i = 1 : length(files)
    disp(['loading file ' int2str(i)]);
    [skeleton, channels, framelength] = bvhReadFile([files{i}]);
    wave_motions_a(i).skel =  skeleton;
    wave_motions_a(i).bvhdata =  channels;
    wave_motions_a(i).framelength =  framelength;
    wave_motions_a(i).filename = files{i};
    wave_motions_a(i).quaternions = f_calculate_quaternions(channels);
    
    % Here are two versions of coordinate calculation. The first is slow and
    % general. The second is faster, but more picky about the skeleton structure.
    
    %wave_motions_a(i).coordinates = f_bvh_to_coordinates(channels, skeleton);
    wave_motions_a(i).coordinates = f_fast_bvh_to_coordinates(channels, skeleton);
end

%% Save data

save('matlab_data/waves_1.mat', 'wave_motions_a');
disp('Saved data to disk.');

%% Animating an example motion

f_animate(wave_motions_a(5).bvhdata, wave_motions_a(5).skel, wave_motions_a(5).coordinates, 'figNum', 100);
