clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');
addpath('./external_libraries/slerp');

%% Creating a new motion

load('matlab_data/waves_1.mat');

from_index = 32;
to_index = 35;
percentage = 0.5;

bvhdata = f_interpolation_pairwise(wave_motions_a(from_index), wave_motions_a(to_index), percentage);
coordinates = f_fast_bvh_to_coordinates(bvhdata(1:end, :), wave_motions_a(1).skel);

%% An example of animating and writing a video

% If you pan, rotate or zoom the figure when writing a video, those actions
% will show in the video.

% The allowed video formats depend on which operating system you are
% running. Type 'help writeVideo' for more details. You might want to
% compress the produced video with a program like ffmpeg.

f_animate(bvhdata, wave_motions_a(from_index).skel, coordinates, ...
    'videoName', 'my_video.avi', ...
    'skeletonQuality', 2, ...
    'figureSize', [800 600]);

%% An example of saving a video with a different rate

% As the original motion has 100 fps, skipping every other frame will give
% 50 fps. However, the 'trajectoryData' is best to be the original 100 fps
% version as the lower fps data may show sharp turns.

f_animate(bvhdata(1:2:end, :), wave_motions_a(from_index).skel, coordinates(1:2:end, :), ...
    'videoName', 'my_video_at_50_fps.avi', 'fps', 50, ...
    'trajectoryData', coordinates);

%% An example of animating and writing the frames to disk one by one

% If move your mouse cursor in front of the figure when the individual
% frames are being captured,  the cursor will also be saved to the disk.

f_animate(bvhdata, wave_motions_a(from_index).skel, coordinates, 'saveFrames', true);
